/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Raúl Yeguas <neokore@gmail.com>
 *              Martin Borho <martin@borho.net>
 */
import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1

Component {
    ComposerSheet {
        id: sheet
        objectName: "SettingsSheet"
        title: i18n.tr("Settings")
        contentsHeight: parent.height

        container: Column {
            anchors.left: parent.left
            anchors.right: parent.right

            Component {
                id: unitsSelectorDelegate
                OptionSelectorDelegate { 
                    text: i18n.tr(label)
                    objectName: name 
                }
            }

            ListModel {
                id: temperatureUnitsSelectorModel
                ListElement { name: "celsiusOption"; label: "Celsius";  }
                ListElement { name: "fahrenheitOption"; label: "Fahrenheit";  }
            }

            OptionSelector {
                id: temperatureUnitsSelector
                objectName: "TemperatureUnitsSelector"
                text: i18n.tr("Temperature units")
                delegate: unitsSelectorDelegate
                model: temperatureUnitsSelectorModel
                selectedIndex: (settings["units"] === "imperial") ? 1 : 0;
            }

            ListModel {
                id: windUnitsSelectorModel
                ListElement { name: "kmhOption"; label: "Kilometers per hour" }
                ListElement { name: "mphOption"; label: "Miles per hour" }
            }

            OptionSelector {
                id: windUnitsSelector
                objectName: "WindUnitsSelector"
                text: i18n.tr("Wind speed units")
                delegate: unitsSelectorDelegate
                model: windUnitsSelectorModel
                selectedIndex: (settings["wind_units"] === "mph") ? 1 : 0;
            }

            ListModel {
                id: precipitationUnitsSelectorModel
                ListElement { name: "millimetersOption"; label: "Millimeters" }
                ListElement { name: "inchesOption"; label: "Inches" }
            }

            OptionSelector {
                id: precipitationUnitsSelector
                objectName: "PrecipitationUnitsSelector"
                text: i18n.tr("Precipitation units")
                delegate: unitsSelectorDelegate
                model: precipitationUnitsSelectorModel
                selectedIndex: (settings["precip_units"] === "in") ? 1 : 0;
            }
        }

        Component.onCompleted: {
            Theme.palette.selected.backgroundText = "#656565"
        }

        onConfirmClicked: {
            var refresh = false,
                selectedUnit = (temperatureUnitsSelector.selectedIndex === 0) ? "metric" : "imperial",
                selectedWindUnit = (windUnitsSelector.selectedIndex === 0) ? "kmh" : "mph",
                selectedPrecipUnit = (precipitationUnitsSelector.selectedIndex === 0) ? "mm" : "in";
            // check if temperaure scale was changed
            if(settings["units"] !== selectedUnit) {
                storage.saveSetting("units", selectedUnit);
                refresh = true;
            }
            //
            if(settings["wind_units"] !== selectedWindUnit) {
                storage.saveSetting("wind_units", selectedWindUnit);
                refresh = true;
            }
            //
            if(settings["precip_units"] !== selectedPrecipUnit) {
                storage.saveSetting("precip_units", selectedPrecipUnit);
                refresh = true;
            }
            // handling of other settings here
            // ....

            // a setting was changed, reload settings and refresh the location tabs
            if(refresh === true) {
                storage.getSettings(function(storedSettings) {
                    for(var settingName in storedSettings) {
                        settings[settingName] = storedSettings[settingName];
                    }
                    refreshData(true);
                });
            }
            Theme.palette.selected.backgroundText = "#f4f4e8"
            PopupUtils.close(sheet)
        }

        onCancelClicked: {
            Theme.palette.selected.backgroundText = "#f4f4e8"
        }
    }
}
