.pragma library
/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Raúl Yeguas <neokore@gmail.com>
 *              Martin Borho <martin@borho.net>
 *              Andrew Starr-Bochicchio <a.starr.b@gmail.com>
 */

/**
*  Version of the response data format.
*  Increase this number to force a refresh.
*/
var RESPONSE_DATA_VERSION = 20130927;

/**
* Helper functions
*/
function calcFahrenheit(celsius) {
        return celsius * 1.8 + 32;
}
//
function calcMph(ms) {
    return ms*2.24;
}
//
function calcInch(mm) {
    return mm/25.4;
}
//
function calcKmh(ms) {
    return ms*3.6;
}
//
function calcWindDir(degrees) {
    var direction =  "?";
    if(degrees >=0 && degrees <= 30){
        direction = "N";
    } else if(degrees >30 && degrees <= 60){
        direction = "NE";
    } else if(degrees >60 && degrees <= 120){
        direction = "E";
    } else if(degrees >120 && degrees <= 150){
        direction = "SE";
    } else if(degrees >150 && degrees <= 210){
        direction = "S";
    } else if(degrees >210 && degrees <= 240){
        direction = "SW";
    } else if(degrees >240 && degrees <= 300){
        direction = "W";
    } else if(degrees >300 && degrees <= 330){
        direction = "NW";
    } else if(degrees >330 && degrees <= 360){
        direction = "N";
    }
    return direction;
}

//
function getLocationTime(tstamp) {
    var locTime = new Date(tstamp);
    return {
        year: locTime.getUTCFullYear(),
        month: locTime.getUTCMonth(),
        date: locTime.getUTCDate(),
        hours: locTime.getUTCHours(),
        minutes: locTime.getUTCMinutes()
    }
}

var GeoipApi = (function() {
    var _baseUrl = "http://geoip.ubuntu.com/lookup";
    return {
        getLatLong: function(params, apiCaller, onSuccess, onError) {
            var request = { type: "geolookup",url: _baseUrl},
                resultHandler = (function(request, xmlDoc) {
                    var coords = {},
                        childNodes = xmlDoc.childNodes;
                    for(var i=0;i<childNodes.length;i++) {
                        if(childNodes[i].nodeName === "Latitude") {
                            coords.lat = childNodes[i].firstChild.nodeValue;
                        } else if(childNodes[i].nodeName === "Longitude") {
                            coords.lon = childNodes[i].firstChild.nodeValue;
                        }
                    }
                    onSuccess(coords);
                }),
                retryHandler = (function(err) {
                    console.log("geolookup retry of "+err.request.url);
                    apiCaller(request, resultHandler, onError);
                });
            apiCaller(request, resultHandler, retryHandler);
        }
    }
})();

var GeonamesApi = (function() {
    /**
      provides neccessary methods for requesting and preparing data from Geonames.org
    */
    var _baseUrl = "http://api.geonames.org/";
    var _username = "uweatherdev"
    var _addParams = "&maxRows=25&featureClass=P"
    //
    function _buildSearchResult(request, data) {
        var searchResult = { locations: [], request: request };
        if(data.geonames) {
            data.geonames.forEach(function(r) {
               searchResult.locations.push({
                    name: r.name,
                    coord: {lat: r.lat, lon: r.lng},
                    country: r.countryCode,
                    countryName: r.countryName,
                    timezone: r.timezone,
                    adminName1: r.adminName1,
                    adminName2: r.adminName2,
                    adminName3: r.adminName3,
                    population: r.population,
                    services: {
                        "geonames": r.geonameId
                    }
                });
            })
        }
        return searchResult;
    }
    //
    return {
        //
        search: function(mode, params, apiCaller, onSuccess, onError) {
            var request,
                retryHandler = (function(err) {
                        console.log("search retry of "+err.request.url);
                        apiCaller(request, searchResponseHandler, onError);
                }),
                searchResponseHandler = function(request, data) {
                    onSuccess(_buildSearchResult(request, data));
                };
            if(mode === "point") {
                request = { type: "search",
                            url: _baseUrl+ "findNearbyPlaceNameJSON?style=full&username="+encodeURIComponent(_username)
                            +"&lat="+encodeURIComponent(params.coords.lat)+"&lng="+encodeURIComponent(params.coords.lon)
                            +_addParams}
            } else {
                request = { type: "search",
                            url: _baseUrl+ "searchJSON?style=full&username="+encodeURIComponent(_username)
                                    +"&name_startsWith="+encodeURIComponent(params.name)+_addParams}
            }
            apiCaller(request, searchResponseHandler, retryHandler);
        }
    }

})();

var OpenWeatherMapApi = (function() {
    /**
      provides neccessary methods for requesting and preparing data from OpenWeatherMap.org
    */
    var _baseUrl = "http://api.openweathermap.org/data/2.5/";
    //
    var _icon_map = {
        "01d": "clear_day",
        "01n": "clear_night",
        "02d": "few_clouds_day",
        "02n": "few_clouds_night",
        "03d": "scattered_clouds_day",
        "03n": "scattered_clouds_night",
        "04d": "broken_clouds_day",
        "04n": "broken_clouds_night",
        "09d": "shower_rain_day",
        "09n": "shower_rain_night",
        "10d": "rain_day",
        "10n": "rain_night",
        "11d": "thunderstorm_day",
        "11n": "thunderstorm_night",
        "13d": "snow_day",
        "13n": "snow_night",
        "50d": "mist_day",
        "50n": "mist_night"
    }    
    //
    function _buildDataPoint(date, data) {
        var result = {
            timestamp: data.dt,
            date: date,
            metric: {
                temp:data.main.temp,
                windSpeed: calcKmh(data.main.speed),
                rain: data.main.rain || 0,
                snow: data.main.snow || 0
            },
            imperial: {
                temp: calcFahrenheit(data.main.temp),
                windSpeed: calcMph(data.main.speed),
                rain: calcInch(data.main.rain || 0),
                snow: calcInch(data.main.snow || 0)
            },
            humidity: data.main.humidity,
            pressure: data.main.pressure,
            windDeg: data.main.deg,
            windDir: calcWindDir(data.main.deg),
            icon: _icon_map[data.weather[0].icon],
            condition: data.weather[0]
        };
        if(data.id !== undefined) {
            result["service"] = "openweathermap";
            result["service_id"] =  data.id;
        }
        return result;
    }
    //
    function _buildDayFormat(date, data) {
        var result = {
            date: date,
            timestamp: data.dt,
            metric: {
                tempMin: data.temp.min,
                tempMax: data.temp.max,
                windSpeed: calcKmh(data.speed),
                rain: data.rain || 0,
                snow: data.snow || 0
            },
            imperial: {
                tempMin: calcFahrenheit(data.temp.min),
                tempMax: calcFahrenheit(data.temp.max),
                windSpeed: calcMph(data.speed),
                rain: calcInch(data.rain || 0),
                snow: calcInch(data.snow || 0)
            },
            pressure: data.pressure,
            humidity: data.humidity,
            icon: _icon_map[data.weather[0].icon],
            condition: data.weather[0],
            windDeg: data.deg,
            windDir: calcWindDir(data.deg),
            hourly: []
        }
        return result;
    }
    //
    function formatResult(data, location) {
        var tmpResult = {},
            result = [],
            day=null,
            offset=(location.timezone && location.timezone.gmtOffset) ? location.timezone.gmtOffset*60*60*1000: 0,
            localNow = getLocationTime(new Date().getTime()+offset),
            todayDate;        
        print("["+location.name+"] "+JSON.stringify(localNow))
        // add openweathermap id for faster responses
        if(location.services && !location.services["openweathermap"] && data["current"].id) {
            location.services["openweathermap"] = data["current"].id
        }
        //
        data["daily"]["list"].forEach(function(dayData) {
            var date = getLocationTime(((dayData.dt*1000)-1000)+offset), // minus 1 sec to handle +/-12 TZ
                day = date.year+"-"+date.month+"-"+date.date;
            if(!todayDate) {
                if(localNow.year+"-"+localNow.month+"-"+localNow.date > day) {
                    // skip "yesterday"
                    return;
                }
                todayDate = date;
            }
            tmpResult[day] = _buildDayFormat(date, dayData);
        })
        //
        var today = todayDate.year+"-"+todayDate.month+"-"+todayDate.date
        tmpResult[today]["current"] = _buildDataPoint(todayDate, data["current"]);
        if(data["forecast"] !== undefined) {
            data["forecast"]["list"].forEach(function(hourData) {                
                var dateData = getLocationTime((hourData.dt*1000)+offset),
                    day = dateData.year+"-"+dateData.month+"-"+dateData.date;
                if(tmpResult[day]) {
                    tmpResult[day]["hourly"].push(_buildDataPoint(dateData, hourData));
                }
            })
        }
        //
        for(var d in tmpResult) {
            result.push(tmpResult[d]);
        }
        return result;
    }
    //
    function _getUrls(params) {
        var urls = {
                current: "",
                daily: "",
                forecast: ""
            },
            latLongParams = "&lat="+encodeURIComponent(params.location.coord.lat)
                + "&lon="+encodeURIComponent(params.location.coord.lon);
        if(params.location.services && params.location.services["openweathermap"]) {
            urls.current = _baseUrl + "weather?units="+params.units+"&id="+params.location.services["openweathermap"];
            urls.daily = _baseUrl + "forecast/daily?id="+params.location.services["openweathermap"]+"&cnt=10&units="+params.units
            urls.forecast = _baseUrl + "forecast?id="+params.location.services["openweathermap"]+"&units="+params.units

        } else if (params.location.coord) {
            urls.current = _baseUrl + "weather?units="+params.units+latLongParams;
            urls.daily = _baseUrl+"forecast/daily?cnt=10&units="+params.units+latLongParams;
            urls.forecast = _baseUrl+"forecast?units="+params.units+latLongParams;
        }
        return urls;
    }
    //
    return {     
        //
        getData: function(params, apiCaller, onSuccess, onError) {
            var urls = _getUrls(params),
                handlerMap = {
                current: { type: "current",url: urls.current},
                daily: { type: "daily",url: urls.daily},
                forecast: { type: "forecast", url: urls.forecast}},
            response = {
                location: params.location,
                db: (params.db) ? params.db : null,
                format: RESPONSE_DATA_VERSION
            },
            respData = {},
            addDataToResponse = (function(request, data) {
                var formattedResult;
                respData[request.type] = data;
                if(respData["current"] !== undefined
                        && respData["forecast"] !== undefined
                            && respData["daily"] !== undefined) {
                    response["data"] = formatResult(respData, params.location)
                    onSuccess(response);
                }
            }),
            onErrorHandler = (function(err) {
                onError(err);
            }),
            retryHandler = (function(err) {
                console.log("retry of "+err.request.url);
                var retryFunc = handlerMap[err.request.type];
                apiCaller(retryFunc, addDataToResponse, onErrorHandler);
            });
            //
            apiCaller(handlerMap.current, addDataToResponse, retryHandler);
            apiCaller(handlerMap.forecast, addDataToResponse, retryHandler);
            apiCaller(handlerMap.daily, addDataToResponse, retryHandler);
        }
    }

})();

var WeatherApi = (function(_services) {
    /**
      proxy for requesting weather apis, the passed _services are providing the respective api endpoints
      and formatters to build a uniform response object
    */
    function _getService(name) {
        if(_services[name] !== undefined) {
            return _services[name];
        }
        return _services["openweathermap"];
    }
    //
    function _sendRequest(request, onSuccess, onError) {
        var xmlHttp = new XMLHttpRequest();
        if (xmlHttp) {
            console.log(request.url);
            xmlHttp.open('GET', request.url, true);
            xmlHttp.onreadystatechange = function () {
                try {
                    if (xmlHttp.readyState == 4) {
                        if(xmlHttp.status === 200) {
                            if(xmlHttp.responseXML) {
                                onSuccess(request, xmlHttp.responseXML.documentElement);
                            } else {
                                var json = JSON.parse(xmlHttp.responseText);
                                onSuccess(request,json);
                            }
                        } else {
                            onError({
                                msg: "wrong response http code, got "+xmlHttp.status,
                                request: request
                            });
                        }
                    }
                } catch (e) {
                    onError({msg: "wrong response data format", request: request});
                }
            };
            xmlHttp.send(null);
        }
    }
    //
    return  {
        //
        geoLookup: function(params, onSuccess, onError) {
            var service = _getService('geoip'),
                geoNameService = _getService('geonames'),
                lookupHandler = function(data) {
                    print("Geolookup: "+JSON.stringify(data))
                    geoNameService.search("point", {coords:data}, _sendRequest, onSuccess, onError);
                };
            service.getLatLong(params, _sendRequest, lookupHandler, onError)
        },
        //
        search: function(mode, params, onSuccess, onError) {
            var service = _getService('geonames');
            service.search(mode, params, _sendRequest, onSuccess, onError);
        },
        //
        getLocationData: function(params, onSuccess, onError) {
            var service = _getService();
            service.getData(params, _sendRequest, onSuccess, onError);
        },
    }
})({
    "openweathermap": OpenWeatherMapApi,
    "geonames": GeonamesApi,
    "geoip": GeoipApi
});

/**
*  following WorkerScript handles the data requests against the weather API.
*  "message" requires a "params" property with the required params to perform
*  the API call and an "action" property, which will be added also to the response.
*/
if(typeof WorkerScript != "undefined") {
    WorkerScript.onMessage = function(message) {
        // handles the response data
        var finished = function(result) {
            WorkerScript.sendMessage({
                action: message.action,
                result: result
            })
        }
        // handles errors
        var onError = function(err) {
            console.log(JSON.stringify(err, null, true));
            WorkerScript.sendMessage({ 'error': err})
        }
        // keep order of locations, sort results
        var sortDataResults = function(locA, locB) {
            return locA.db.id - locB.db.id;
        }
        // perform the api calls
        if(message.action === "searchByName") {
            WeatherApi.search("name", message.params, finished, onError);
        } else if(message.action === "searchByPoint") {
            WeatherApi.search("point", message.params, finished, onError);
        } else if(message.action === "getGeoIp") {
            WeatherApi.geoLookup(message.params, finished, onError);
        } else if(message.action === "updateData") {
            var locLength = message.params.locations.length,
                locUpdated = 0,
                result = [],
                now = new Date().getTime();
            if(locLength > 0) {
                message.params.locations.forEach(function(loc) {
                    var updatedHnd = function (newData, cached) {
                            locUpdated += 1;
                            if(cached === true) {
                                newData["save"] = false;
                            } else {
                                newData["save"] = true;
                                newData["updated"] =  new Date().getTime();
                            }
                            result.push(newData);
                            if(locUpdated === locLength) {
                                result.sort(sortDataResults);
                                finished(result);
                            }
                        },
                        params = {
                            location:loc.location,
                            db: loc.db,
                            units: 'metric'
                        },
                        secsFromLastFetch = (now-loc.updated)/1000;
                    //
                    if( message.params.force===true || loc.format !== RESPONSE_DATA_VERSION || secsFromLastFetch > 1800){
                        // data older than 30min, location is new or data format is deprecated
                        WeatherApi.getLocationData(params, updatedHnd, onError);
                    } else {
                        console.log("["+loc.location.name+"] returning cached data, time from last fetch: "+secsFromLastFetch)
                        updatedHnd(loc, true);
                    }
                })
            } else {
                finished(result);
            }
        }
    }
}
