This document provides an overview of the subset of commands from the The Weather Channel API (TWC) that the Ubuntu Weather App uses to retrieve weather data.

## General command usage

General usage (URL to query data from TWC):

<server>/wxdata/{subContext}/<command>/<location>{:locType}<format>?{parameters&<apikey>&{locale}}

* Server: http://wxdata.weather.com
* SubContext: Optional for commands that are shared between all Subcontexts.
* Command: Any valid command from the Commands section.
* Location: Any valid Type1 or Type4 Location
* locType: Optional, any of the valid loc Types listed in the chart below.
* Format: XML or JSON. The Ubuntu Weather App uses JSON, thus this value is ".js"
* API Key: A valid API key is necessary to access the TWC data. Contact david.planella@canonical.com for any queries related to API key usage.
* Locale: Standared Locale codes, such as es_ES, de_DE, and pt_BR.

Location Type: Description
1: Cities
4: Postal codes
5: Golf Courses
9: Airports
11: Ski resorts
13: Recreation Locations
16: Intersections
17: Schools
18: Shopping Centers
19: Outdoor Locations
20: Event Venues
21: Hydrography Locations (lakes and rivers)

Dates are expressed in GMT epoch seconds
Example: 1325419200 = GMT: Sun, 1 Jan 2012 12:00:00 UTC

### Alternative command usage

Where appropriate, commands can be requested using latitude and longitude coordinates.

<server>/wxdata/<command>/get.<format>?<lat=latitude><lng=longitude>

* lat: Latitude
* lng: Longitude

Example: http://wxdata.weather.com/wxdata/loc/get.js?lat=33.68&lng=-84.23

## Units of measure

The following Units of measure are supported:

* English Units (Default)
* Metric Units
* Hybrid (User Specified mix of English and Metric Units)

The Ubuntu Weather App supports only English (Imperial) Units or Metric Units, determined by a configurable setting.

Specify the units as parameters: "units=e" for imperial and "units=m" for metric.

Example: http://wxdata.weather.com/wxdata/loc/get.js?lat=33.68&lng=-84.23&units=m

## Commands

### Location

Usage:
/loc/{location}.{output format}?{url parameters}

Returns additional location details for the supplied location. The Ubuntu Weather App uses this command to retrieve location IDs and other details for a given set of latitude and longitude coordinates.

Lifetime of Data: 24 hours

URL Parameters:
* key: (required) A valid API Key.

Example URLs:
* http://wxdata.weather.com/wxdata/loc/12345.js?key=<apikey>
* http://wxdata.weather.com/wxdata/loc/get.js?key=<apikey>&lat=42.810001373291016&lng=-73.94
* http://wxdata.weather.com/wxdata/loc/30126:4.js?country=FR&locale=de_DE&key=<api_key>
* http://wxdata.weather.com/wxdata/loc/30126:4.js?country=US&locale=de_DE&key=<api_key>

Output Object: Location

### Location Search

Usage:
/locsearch/{search String}.{output format}?{url parameters}

Searches for locations matching the term passed in. The Ubuntu Weather App uses this command to enable the user searching and setting a location and to retrieve the weather data for that location.

Lifetime of Data: 24 hours

URL Parameters:
* key: (required) A valid API Key.
* max: The Max number of locations returned
* loctypes: Comma-delimited list of loc types to search. Default '1'

Example URLs:
* http://wxdata.weather.com/wxdata/locsearch/london.js?key=<apikey>&locale=es_ES

Output Object: List [ LocationScoreResult ]

### Mobile aggregation

Usage:
/mobile/mobagg/{location(s)}.{output format}?{url parameters}

Returns the following data for each location(s). Comma-delimited list of locations/location types are supported. The Ubuntu Weather App uses this command to retrieve daily and hourly forecast data.

* Data Returned: (Location, Daily Forecast, Hourly Forecast, HourlyHirad Forecast, Narrative Forecast, SunRiseSet Collection - 10 day, HiradObservation Data, StandardObservation Data, Severe Weather Alerts, pollen, hirad now, and boat & Beach)

Lifetime of Data: TBD

URL Parameters:
* key:(required) A valid API Key.
* validLocationTypes:
  * Valid values: 19, 4, 11, 9, 13, 16, 5, 21, 1, 17, 20, 18, -1
* roles:
  * Valid Values: hirad
* hours: Desired number of hours from 1 to 48. Default is 24

Example URLs:
* http://wxdata.weather.com/wxdata/mobile/mobagg/atl:9,lax:9.js?key=<apikey>

### Observation

Usage:
/obs/{location}.{output format}

Observation (Current Condition) for the specified location. The Ubuntu Weather App uses this command to obtain the current barometric pressure and precipitation values.

Lifetime of Data: 5 Minutes

URL Parameters:
* key:(required) A valid API Key.

Example URLs:
* http://wxdata.weather.com/wxdata/obs/get.js?key=<apikey>&lat=42.810001373291016&lng=-73.94&day=all&units=m

Output Object: Observation

## Output Objects

### Location

* key : String : The TWC Loc Id
* type : Int : The Location Type
* countryCode : String : The Country Code
* lat : Double : The Location's Latitude
* lng : Double : The Location's Longitude
* elv : Int : The Location's Elevation
* tz : Double : The Location's Timezone
* zoneInfo : String : The full name of the Timezone
* name : String : The Location's name
* city : String : The nearest City's Name
* state : String : The State Code
* country : String : The Country Name
* forecastID : String : The Observation Station for the location
* dma : String : The DMA for the location
* rgn4 : String : The large scale radar mapping
* rgn9 : String : The small scale radar mapping
* zoneId : String : The NWS Alert Zone for the location
* countyId : String : The county Id for the location
* isMarineSurf : String : If the location is Coastal with a Boat and Beach report
* zip : String : The Zip code of the location
* parentLoc : String : The nearest Type 1 Location matched to this location
* cID : String : The Claritas Id for the location

### LocationScoreResult

* loc : Location : The Location Data
* score : Double : The Location's score, with 1.0 being a perfect match

### Observation

* key : String : The Observation Station
* visibility : Double : The Visibility
* date : Epoch : The GMT time of the observation
* wxIcon : Int : The Weather Icon
* temp : Int : The current temp
* tempMax24 : Int : The Maximum expected temperature for the next 24 hrs
* tempMin24 : Int : The Minimum expected temperature for the next 24 hrs
* precip24Amount : Int : The amount of Precipitation expected in the next 24 hours
* dewPoint : Int : The Dew Point
* pressure : Double : The Barometric Pressure
* pressureDescription : String : The Barometric Tendency Description
* feelsLike : Int : The Feels Like Temp
* humid : Int : The Relative Humidity
* wSpeed : Int : The Wind Speed
* wGust : Int : The Wind Gust SpeedwDir : Int : The Wind Direction, 0- 360
* tend : Int : The Barometric Tendency
* uv : Int : The UV Index
* uvText : String : The UV Index description
* obsName : String : The name of the Observation Station
* wDirText : String : The Wind Direction Text
* text : String : The Weather Description
* heatIndex : Int : The Heat Index
* windchill : Int : The Wind Chill
* qualifier : String : The Qualifier
* qualifier_severity : String : The Qualifier Severity
* blunt_phrase : String : The Blunt Phrase description of the Qualifier
* terse_phrase : String : The Terse Phrase description of the Qualifier
* precipHourly : String : The Expected Hourly Precipitation
* snowIncr : String : The Expected Snow Increment
