# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# Authored by: Martin Borho <martin@borho.net>

"""Tests for Weather app"""

from __future__ import absolute_import

from testtools.matchers import Equals, Is, Not
from autopilot.matchers import Eventually

import logging

from ubuntu_weather_app.tests import WeatherTestCase, DatabaseMixin
from ubuntu_weather_app.tests.weatherdata import locations_data

logger = logging.getLogger(__name__)

class TestMainView(WeatherTestCase, DatabaseMixin):
    """Tests creating weather"""

    """ This is needed to wait for the application to start.
        In the testfarm, the application may take some time to show up."""
    def setUp(self):
        #we want to start with fake location data
        self.create_blank_db()
        logger.debug("Adding fake data to new database")
        self.add_locations_to_database(locations_data)
        logger.debug("Re-Launching app to introspect")
        super(TestMainView, self).setUp()
        self.assertThat(
            self.get_qml_view().visible, Eventually(Equals(True)))

    def tearDown(self):
        super(TestMainView, self).tearDown()

    def test_switch_tabs(self):
        """Test start of app with two locations and switch from first to
        second tab"""

        # first tab should be visible after start
        tabs =  self.main_window.get_tabs()
        first_tab = self.main_window.select_single(
            'LocationTab', index=0)
        second_tab = self.main_window.select_single(
            'LocationTab', index=1)
        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(0)))
        self.assertThat(first_tab.visible, Eventually(Equals(True)))
        self.assertThat(second_tab.visible, Eventually(Equals(False)))
        self.assertThat(first_tab.title, Eventually(Equals("Hamburg")))
        # switch to next tab
        self.main_window.switch_to_tab_by_index(1)
        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(1)))
        self.assertThat(first_tab.visible, Eventually(Equals(False)))
        self.assertThat(second_tab.visible, Eventually(Equals(True)))
        self.assertThat(second_tab.title, Eventually(Equals("London")))

    def test_refresh_tabs(self):
        # get the dates from the test data
        self.assertThat(lambda: self.main_window.select_many('Label', objectName='DayDateLabel'), Eventually(Not(Is(None))))
        curr_dates = self.main_window.select_many('Label', objectName='DayDateLabel')
        tab1_curr_date = curr_dates[0].text
        tab2_curr_date = curr_dates[2].text

        # refresh and wait till loading has finished
        self.main_window.open_toolbar().click_button("RefreshButton")
        load_indicator = self.main_window.select_single('ActivityIndicator', objectName='LoadingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False), timeout=120))

        # get the data from the refreshed tabs, have to be new
        self.assertThat(lambda: self.main_window.select_many('Label',objectName='DayDateLabel'), Eventually(Not(Is(None))))
        refreshed_dates = self.main_window.select_many('Label',objectName='DayDateLabel')
        self.assertNotEqual(tab1_curr_date, refreshed_dates[0].text)
        self.assertNotEqual(tab2_curr_date, refreshed_dates[2].text)
